# Copyright © Amazon Web Services
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
from unittest import TestCase
from unittest.mock import patch, Mock
from bundles.providers.aws.storage import AWSBundleStorageClient, FileNotFound, logger


class TestAWSBundleStorageClient(TestCase):
    def setUp(self):
        self.expected_bucket_name = "test-bucket"
        self.mock_client = Mock()
        self.mock_s3_client = Mock()
        self.file_mock = Mock()
        self.test_filename = "test-file"
        self.expected_uri = f"s3://{self.expected_bucket_name}/{self.test_filename}"

        patches = [
            patch(
                "osdu_api.providers.blob_storage.get_client",
                return_value=self.mock_client,
            ),
            patch("boto3.session.Session.client", return_value=self.mock_s3_client),
            patch.dict(
                "os.environ",
                {"POLICY_BUCKET": self.expected_bucket_name, "CLOUD_PROVIDER": "aws"},
            ),
        ]
        for p in patches:
            p.start()
            self.addCleanup(p.stop)

        self.storage_client = AWSBundleStorageClient()

    def test_get_bucket_uri(self):
        expected_uri = f"s3://{self.expected_bucket_name}/{self.test_filename}"
        self.assertEqual(
            self.storage_client._get_bucket_uri(self.test_filename), expected_uri
        )

    def test_upload_file_success(self):
        result = self.storage_client.upload_file(self.test_filename, self.file_mock)

        self.assertEqual(result, self.expected_uri)

    def test_FileNotFound_initialization(self):
        expected_message = f"ERROR: File {self.test_filename} was not found."

        exception = FileNotFound(self.test_filename)

        self.assertEqual(str(exception), expected_message)

    def test_download_file_success(self):
        expected_file_content = "file content"
        self.mock_client.download_file.return_value = expected_file_content
        self.mock_client._does_file_exist.return_value = True

        result_file, result_uri = self.storage_client.download_file(
            self.test_filename, self.file_mock
        )

        self.assertEqual(result_file, self.file_mock)
        self.assertEqual(result_uri, self.expected_uri)

    def test_split_s3_path(self):
        test_cases = [
            (
                "s3://my-bucket/my-folder/my-file.txt",
                ("my-bucket", "my-folder/my-file.txt"),
            ),
            ("s3://another-bucket/", ("another-bucket", "")),
            (
                "s3://bucket/with/long/path/file.txt",
                ("bucket", "with/long/path/file.txt"),
            ),
        ]

        for s3_path, expected in test_cases:
            with self.subTest(s3_path=s3_path):
                self.assertEqual(self.storage_client._split_s3_path(s3_path), expected)

    @patch("bundles.providers.aws.storage.AWSBundleStorageClient._split_s3_path")
    def test_download_to_file(self, mock_split_s3_path):
        mock_object_name = "some/path/to/file.txt"

        mock_split_s3_path.return_value = (self.expected_bucket_name, mock_object_name)

        self.storage_client._download_to_file(self.expected_uri, self.file_mock)

        mock_split_s3_path.assert_called_with(self.expected_uri)
        self.storage_client.s3_client.download_fileobj.assert_called_with(
            self.expected_bucket_name, mock_object_name, self.file_mock
        )
